# R scripts for lesson 1
# basic manipulation with data.frame and descriptive statistics

# when we are not using R Studio, always check path using command:

getwd()

# we can set path using setwd("cesta/k/adresari")

# R has modular architecture - most of the useful functions should be explicitly loaded
# we can load libraries using command library(NAME)

library(psych) # some useful functions for descriptive statistics are there
# in case that package is missing, use following command
# install.packages("psych")
library(ggplot2) # nice plots
theme_set(theme_bw()) # for white background

########################
#   data types in R    #
########################
# we will frenquently use vectors
x<-c(1,2,3) # or alternatively
x<-1:3
x # or print(x)
# note: using = instead of <- works in most cases as well

# matrix
m<-matrix(c(1,2,3,4),nrow=2)
m

# list - can contain any objects
l<-list(age=54,gender="F",m)
l
# data.frame - columns can have arbitrary type, each row is one observation. We will use it later

###########################
#       load data         #
###########################

# all libraries in R have some example data sets, we do not need to load anything, command "data()" will list available datasets. We will load following dataset
data(USArrests)
# rename it
df=USArrests

# and remove object USArrest from workspace
rm(USArrests)

# look at the data
dim(df)
str(df)
head(df)

summary(df)

# we can see that:
# - df has 4 rows and 50 observations (rows)
# - all columns contains numbers, Murder and Rape are with floating point (numeric) and UrbanPop and Assault are integers (we can see this using command str(df))

# there is also a help for this data set (it contains some explanation of variables)
?USArrests

# We can see that this table contains violent crimes in USA in 1973 per 100 000 inhabitants
# column UrbanPop contains percentage of urbanpop

###########################
# descriptive statistics  #
###########################

# using function describe

(desc.stat<-describe(df)) # using parenthesis stores data into the variable and output it as well
# function describe has some parameters influencing output. For most of the analysis, we could use default ones
# In case we are not interested in skewness, we can disable it using
# describe(df,skew=F)

# Usually, we have to report mean, sd, min, max a range. All of that is present in the output

# desc.stat is also data frame, so we can select specific columns and/or rows
desc.stat[,c(3,4,8,9,10)] # this selects mean, sd, min, max a range

# we can sompute mean or sd using functions mean and sd

mean(df$Murder)
sd(df$Murder)

# in case of missing data, we can use parameter to omit them
mean(df$Murder,na.rm=T)


###########################
# Manipulating data frame #
###########################

# we can arbitrary manipulate with data.framem

# select odd numbers between 1 and 50
odd.rows<-seq(1,50,2)

# select odd rows fro mdata frame
df[odd.rows,]

# additional selections
# - by column names
df[c("Alabama","Texas"),]
# - and select specific column
df[c("Alabama","Texas"),"Rape"]

# - without odd rows
df[-odd.rows,]
# - ordered by number of inh. living in cities
df[order(df$UrbanPop),]
# - ordered by murder count (decreasing)
df[order(df$Murder,decreasing=T),]

# logical conditions
with(df,df[Murder>10,])
with(df,df[Murder>10&Rape<25,])

# or more complicated
with(df,df[(Murder>10&Rape<25)|(Assault>200 & !UrbanPop>60),]) 

## function subset
# usefull is function subset; we can select data using logical conditions very easily
subset(df,Murder>10)
subset(df,Murder>10&Rape<25)
subset(df,(Murder>10&Rape<25)|(Assault>200 & !UrbanPop>60))

# we can select/omit specific columns using subset
subset(df,Murder>10,select="Assault") 
subset(df,Murder>10,select=-UrbanPop) # don't select UrbanPop column
subset(df,select=c(-UrbanPop,-Rape)) # more columns at once

## changing columns in data.frame

# following command creates column IsUsaState and set its value to True
df$IsUsaState<-T # not very usefull, but if we have states from different countries, it could be usefull

# renove column
df$IsUsaState<-NULL
head(df)

# usually, we add column with default value (e.g. df$IsUsaState<-T)
# and then we set correct value for some corresponding rows

# let's add information about their vote outcome
# create column with default value
df$Vote<-"Obama" # better set Obama as default value, because he won:-)

# select states, where Romney won
# quite ugly command, but we need do this only once
states.Romney<-c("Alabama","Alaska","Arizona","Arkansas","Georgia","Idaho","Indiana","Kansas","Kentucky","Louisiana","Mississippi","Missouri","Montana","Nebraska","North Carolina","North Dakota","Oklahoma","South Carolina","South Dakota","Tennessee","Texas","Utah","West Virginia","Wyoming")
df[states.Romney,]$Vote<-"Romney" 

# make it factor. Factor is a usefull data type, which has many builtin functions
df$Vote<-as.factor(df$Vote)

# when we look at our df, we can see that there is new column of type factor with two values
str(df)
head(df)

# we can see differences between groups using function describeBy
# to reduce typing, we will use with environment. using with, we do not have to use $ sign for accessing columns
# without with, the same command is: describeBy(df,group=df$Vote))


with(df,describeBy(df[,colnames(df) %in% c("Murder","Assault","UrbanPop","Rape")],group=Vote)) # zaroven vynechame sloupec Vote, protoze pro nej deskriptivni satistika nedava moc smysl
# little bit overkill, but it introduces %in% operator, which selects all values from right vector

# we can print differences between groups side by side
# as above, but we will add parametr mat=T, which creates one data.frame

df.noUrbanPop<-subset(df,select=-UrbanPop) # we do not need urbanPop, we are interested only in crimes
desc.m<-with(df.noUrbanPop,describeBy(df[,colnames(df) %in% c("Murder","Assault","Rape")],group=Vote,mat=T))

# select only few descriptive stat
desc.m<-desc.m[,c("group1","mean","sd","min","max","range")]

# select Romney and Obama states
desc.Romney<-subset(desc.m,group1=="Romney",select=-group1)
desc.Obama<-subset(desc.m,group1=="Obama",select=-group1)

# transpose this matrix - flip it by its main diagonal
desc.Romney<-t(desc.Romney)
desc.Obama<-t(desc.Obama)

# rename columns to see, if it is about R or O
colnames(desc.Romney)<-c("R.Murder","R.Assault","R.Rape")
colnames(desc.Obama)<-c("O.Murder","O.Assault","O.Rape")

# merge them into one data.frame
desc.m<-cbind(desc.Romney,desc.Obama)

# now we can save it into csv file, open it in excel and do some formatting for publication:-)
write.csv2(desc.m,"romney_vs_obama.csv")

# this manipulation can be done much easier way, but it is for ilustration purposes, how can be data.frames manipulated

#################################
#         Basic plots           #
#################################

# some basic plots in R can be created using commands plot, hist and boxplot

plot(Murder~Rape,df) # this is formula notation, we will see this later. It plots Assault on x axis and Murder on y axis
# alternatively
plot(df$Assault,df$Murder)
# or 
with(df,plot(Assault,Murder))

# we can adjust some parameters
plot(Murder~Assault,df,xlab="Assault crimes",ylab="Murder crimes",main="Scatterplot for Murder-Assault relationship")
# but ggplot (later) has many more options, so why bother with this
# alternatively histogram
hist(df$UrbanPop,xlab="Urban population",breaks=10) # number of breaks can greatly influence number of bins

# we can visualize all pairs of scatterplots
pairs(df)

# plots can be exported using some output function, e.g. png() for png format
png("images/plot1.png")
plot(Murder~Assault,df)
dev.off()

# jen tak na ukazku..

cor(as.matrix(df[,1:4]))

summary((lm(Murder~Assault,df))
summary(aov(Assault~Vote,df))
t.test(Assault~Vote,df)

##################################################
# advanced data.frame manipulation               #
##################################################

# we have currently data in wide format.
# In wide format, all columns form one variable. This is not ideal, because many neat functions uses long format instead (they work with tidy data)
# let's transform it into the data.frame, which has columns:
# "Name of state", "Urbanan Population","Type of crime", "Count per 100 000 people"
# Doing this in excel is boring and time consuming
# In R, we have special package reshape2
# install it using install.packages("reshape2")

# load library
library(reshape2)

# add column with rowname
df$State<-rownames(df)

# one command to convert to long format
df.long<-melt(df,id.vars=c("State","UrbanPop","Vote"),measure.vars=c("Assault","Murder","Rape"),variable.name="Crime",value.name="Count")

# we can use subset function on data in long format (everything is in columns)
# we can easily do regression, ANOVA, contingency tables and mainly plots




#################################
### plots for presentation  #####
#################################

# we do not need to understand this at the moment

p<-qplot(Murder,Assault,col=Vote,data=df,size=2)+stat_smooth(method="lm",se=F,size=1.5)
ggsave("images/example-scatterplot.png",p)

x<-rnorm(5000)
p<-qplot(x,binwidth=0.2,colour = I("darkgreen"), fill = I("white"))
ggsave("images/example-histogram.png",p)

p<-ggplot(df.long,aes(x=Crime,y=Count))+geom_boxplot()
ggsave("images/example-boxplot.png",p)
